import { Either } from "monet";
import { AsyncOrSync, MarkOptional } from "ts-essentials";
import { ObjectTypeComposer, ResolverResolveParams } from "graphql-compose";
import { DataLoaderManager } from '@pallad/dataloader-manager';
import { EntityHelper } from './EntityHelper';
import { Command } from 'alpha-command-bus-core';
export declare class Mapper<TContext = any, TDataLoaderContext = any, TGQLContext extends Mapper.BasicGQLContext<TDataLoaderContext> = any> {
    private commandBusHandler;
    private dataLoaderManager;
    static DEFAULT_OPTIONS: {
        resultHandler(result: Either<any, any>): any;
    };
    private options;
    constructor(commandBusHandler: Mapper.CommandBusHandler<Mapper.CommandBusHandler.Context<TContext, TGQLContext | TDataLoaderContext>>, dataLoaderManager: DataLoaderManager, options: Mapper.Options.FromUser);
    handleCommand(command: Command, context: TContext, gqlContext: TGQLContext | TDataLoaderContext, resultHandler?: Mapper.ResultHandler): Promise<any>;
    createResolver<TArgs = any, TSource = any>(options: Mapper.CreateResolverOptions<TContext, TGQLContext, TArgs, TSource>): Mapper.Resolver<TArgs, TSource, TGQLContext>;
    createDataLoaderResolver(dataLoaderName: string, argKeyName?: string): (rp: ResolverResolveParams<any, TGQLContext>) => Promise<unknown> | undefined;
    createEntityHelper<TEntity = any>(entityType: ObjectTypeComposer<TEntity, TGQLContext>): EntityHelper<TEntity, TGQLContext, TContext, TDataLoaderContext>;
}
export declare namespace Mapper {
    type ResolverFunc<TArgs, TSource, TGQLContext> = (source: TSource, args: TArgs, context: TGQLContext) => Promise<any>;
    interface Resolver<TArgs, TSource, TGQLContext> extends ResolverFunc<TArgs, TSource, TGQLContext> {
        rp: (rp: ResolverResolveParams<TSource, TGQLContext, TArgs>) => Promise<any>;
    }
    interface BasicGQLContext<TDataLoaderContext = any> {
        dataLoaders: DataLoaderManager.Scope<TDataLoaderContext>;
    }
    type CommandBusHandler<TContext extends CommandBusHandler.Context<any, any>> = (command: Command, context: TContext) => Promise<any>;
    namespace CommandBusHandler {
        interface Context<TOptions, TCommonGQLContext> {
            context: TOptions;
            gqlContext: TCommonGQLContext;
        }
    }
    interface Options {
        resultHandler: ResultHandler;
    }
    namespace Options {
        type FromUser = MarkOptional<Options, 'resultHandler'>;
    }
    type ResultHandler = (result: Either<any, any>) => any | Promise<any>;
    interface CreateResolverOptions<TCommandBusContext extends {}, TGQLContext, TArgs extends Record<string, any>, TSource = any> {
        commandFactory: (data: {
            args: TArgs;
            source: TSource;
            context: TGQLContext;
        }) => AsyncOrSync<Command>;
        resultHandler?: ResultHandler;
        context: TCommandBusContext;
    }
}
